import java.util.Random;

/**
 * A program that tests the preformance of the find method for
 * SimpleUSet implementations. The program generates a SimpleUSet of
 * Integers (specifically, the prime the numbers 2, 3, 5, 7, 11, 13,
 * 17, 19,... that are not divisible by any smaller number other than
 * 1), then repeated calls find(n) for random choices of n to test the
 * performance of the find method.
 */

public class PrimeTime {
    /**
     * the maximum value stored in any set
     */
    public static final int MAX_VALUE = 1_000_000;

    /**
     * the step size of maximum values of sets considered
     * e.g., the program will construct sets with maximum values STEP,
     * 2 * STEP, 3 * STEP, ..., MAX_VALUE
     */
    public static final int STEP = 50_000;

    /**
     * the number of calls to find tested for each set size
     */
    public static final int NUM_SAMPLES = 1_000;

    public static void main(String[] args) {
	CSVWriter csv = new CSVWriter("prime-times-uset.csv");
	
	SimpleUSet<Integer> primes; 


	csv.addEntry("max value");
	csv.addEntry("num primes");
	csv.addEntry("time to build (ms)");
	csv.addEntry("time per find (ns)");
	csv.endLine();

	for (int i = STEP; i <= MAX_VALUE; i += STEP) {
	    
	    /*
             ************************************************
	     * pick a SimpleUSet<Integer> implementation here
	     ************************************************
	     */
	    
	    primes = new ArraySimpleUSet<Integer>();
	    //primes = new ArraySimpleSSet<Integer>();
	    
	    timeTest(i, primes, csv);
	}

	csv.close();
    }
    

    static void timeTest(int maxP, SimpleUSet<Integer> primes, CSVWriter csv) {
	Random rand = new Random();
	RunTimer rt = new RunTimer();
	PrimeTester pt = new PrimeTester(maxP);

	int nPrimes = 0;

	// add all primes up to maxP to primes and measure how long it
	// takes to perform the add methods
	for (int i = 2; i < maxP; ++i) {
	    if (pt.isPrime(i)) {
		++nPrimes;
		
		rt.start();
		primes.add(i);
		rt.stop();
	    }
	}

	csv.addEntry(maxP);
	csv.addEntry(nPrimes);
	csv.addEntry(rt.getElapsedMillis());

	rt.reset();

	// pick random numbers and test if they are prime (i.e.,
	// contained in the set primes)
	for (int i = 0; i < NUM_SAMPLES; ++i) {
	    int val = rand.nextInt(maxP);
	    
	    rt.start();
 	    primes.find(val);
	    rt.stop();
	}

	csv.addEntry(rt.getElapsedNanos() / NUM_SAMPLES);
	csv.endLine();
	
    }
    
}
